/*

*/

import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

/**

*/
public class Threads_vs_ThreadPool
{
   public static void main(String[] args) throws InterruptedException
   {
      Runnable[] tasks =  new Runnable[1_000];

      // Instantiate an array of Runnable objects.
      for (int i = 0; i < 1_000; ++i) // try 10_000
      {
         final long x = i + 50_000_000_000_000_001L;
         tasks[i] = new PrimeTask(x);
      }

      // Submit the Runnable objects to a ThreadPool.
      long startTime = System.currentTimeMillis();
      for (Runnable task : tasks)
      {
         executor.execute( task );
      }
      executor.shutdown();
      // Wait for all of them to finish.
      executor.awaitTermination(1, TimeUnit.MINUTES); // throws InterruptedException
      long stopTime = System.currentTimeMillis();
      System.out.println("(" + (stopTime - startTime) + "ms)");

      System.out.println("\n**************************************\n");

      // Use the Runnable objects to create, and run, threads.
      startTime = System.currentTimeMillis();
      Thread[] threads =  new Thread[1_000];
      for (int i = 0; i < tasks.length; ++i)
      {
         threads[i] = new Thread(tasks[i]);
         threads[i].start();
      }
      // Wait for all of them to finish.
      for (Thread thread : threads)
      {
         thread.join(); // throws InterruptedException
      }
      stopTime = System.currentTimeMillis();
      System.out.println("(" + (stopTime - startTime) + "ms)");
   }

   private static final int cores = Runtime.getRuntime().availableProcessors();
   private static final ExecutorService executor = Executors.newFixedThreadPool(cores);
}


class PrimeTask implements Runnable
{
   private long n;

   public PrimeTask(long n)
   {
      this.n = n;
   }

   @Override
   public void run()
   {
      System.out.println( Primes.factorize(n) );
   }
}
