/*
  https://eli.thegreenplace.net/2018/concurrent-servers-part-6-callbacks-promises-and-asyncawait/
*/

import java.util.concurrent.TimeUnit;
import java.util.Random;

/**

*/
public class UnitOfWork
{
   // CPU-bound work.
   public static long doWork(final long time, final TimeUnit unit)
   {
      long ms = TimeUnit.MILLISECONDS.convert(time, unit);
      long awake_time = System.currentTimeMillis() + ms;
      while ( awake_time > System.currentTimeMillis() )
      {
         /* empty */
      }
      return ms;
   }


   // Simulate I/O-bound work.
   public static long fakeWork(final long time, final TimeUnit unit)
   {
      long ms = TimeUnit.MILLISECONDS.convert(time, unit);
      try
      {
         TimeUnit.MILLISECONDS.sleep(ms);
      }
      catch (InterruptedException e){}
      return ms;
   }


   // Simulate a mixture of CPU-bond and I/O-bound work.
   public static long hybridWork(final long time, final TimeUnit unit, double percent)
   {
      long ms = TimeUnit.MILLISECONDS.convert(time, unit);
      doWork( (long)(percent * ms), TimeUnit.MILLISECONDS);
      fakeWork( (long)((1 - percent) * ms), TimeUnit.MILLISECONDS);
      return ms;
   }


   // Similate a random mixture of CPU-bound and I/O bound work.
   public static long syntheticdWork(final long time, final TimeUnit unit)
   {
      long ms = TimeUnit.MILLISECONDS.convert(time, unit);

      Random rn = new Random();
      int units = 1 + rn.nextInt(8);
      int[] cuts = new int[units + 1];
      cuts[0] = 0;
      cuts[units] = 1000;
      for (int i = units - 1; i > 0; --i)
      {
         cuts[i] = 1 + rn.nextInt(cuts[i+1] - 1);
      }

      //System.out.println( java.util.Arrays.toString(cuts) );

      int flip = rn.nextInt(2);

      for (int i = 1; i <= units; ++i)
      {
         long unitTime = (long)(( (cuts[i] - cuts[i-1])/1_000.0 ) * ms);

         //System.out.println("unitTime = " + unitTime + ((i%2==flip)?" (work)":" (sleep)"));

         if (i % 2 == flip)
            doWork(unitTime, TimeUnit.MILLISECONDS);
         else
            fakeWork(unitTime, TimeUnit.MILLISECONDS);
      }

      return ms;
   }
}
